package com.yixin.ms.controller;


import com.alibaba.fastjson.JSON;
import com.yixin.ms.boot.restful.RestResponse;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.model.dto.HealthCodeDTO;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.dto.ThirdPlatformSm4RespDto;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.PlaceHealthCodeServer;
import com.yixin.ms.server.vo.HealthCodeVO;
import com.yixin.ms.server.vo.PlaceInfoVO;
import com.yixin.ms.service.HealthCodeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Map;


@Slf4j
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "健康码查询 API")
@RestController
@RequestMapping("hh")
public class HealthCodeController {
    /**
     * 服务对象
     */
    private final HealthCodeService healthCodeService;
    private final PlaceHealthCodeServer placeHealthCodeServer;

    @ApiOperation("查询")
    @PostMapping("/getHealthCode")
    public RestResult<HealtCodeUserVO> getShortTermCertificate(@RequestBody ShortTermDTO shortTermDTO) {
        return healthCodeService.getShortTerm(shortTermDTO);
    }

    @ApiOperation("创建token")
    @PostMapping("/addToken")
    public RestResult<String> addToken(String token) {
        if (StringUtils.isBlank(token)) {
            throw new ServiceException("token为空哦");
        }
        if (!StringUtils.equals("hanghui", token)) {
            throw new ServiceException("权限不足");
        }
        String shortTermCertificate = healthCodeService.addToken(token);
        if (StringUtils.isNotBlank(shortTermCertificate)) {
            return RestResponse.ok(shortTermCertificate);
        }
        throw new ServiceException(703, "未查询到此数据");
    }

    @ApiOperation("删除token")
    @GetMapping("/deleteToken")
    public RestResult<Boolean> deleteToken(String token) {
        if (StringUtils.isBlank(token)) {
            throw new ServiceException("token为空哦");
        }
        Boolean shortTermCertificate = healthCodeService.deleteToken(token);
        if (shortTermCertificate) {
            return RestResponse.ok(shortTermCertificate);
        }
        throw new ServiceException("删除失败");
    }

    @ApiOperation("根据barcode查询健康码信息")
    @PostMapping("/queryInfoByBarCode")
    public RestResult<HealthCodeVO> queryInfoByBarCode(@RequestBody HealthCodeDTO healthCodeDTO) {
        return healthCodeService.queryInfoByBarCode(healthCodeDTO);
    }

    @ApiOperation("三方平台接口")
    @PostMapping("/getHealthCode2")
    public RestResult<ThirdPlatformSm4RespDto> getShortTermCertificate2(@RequestBody ShortTermDTO shortTermDTO) {
        return RestResponse.ok(new ThirdPlatformSm4RespDto());
    }

    /*
    * 这个接口是为了让设备上原先请求服务商平台再请求，联动平台的逻辑变更为 之请求联动平台
    * 由联动平台去请求服务商平台，再进行对应的业务逻辑
    * 目前我会先在这里做操作
    * 后面将上面的getShortTermCertificate方法的Mapping改到这里就行了
    *
    * */
//    @ApiOperation("查询开门结果")
//    @PostMapping("/getHealthCode")
//    public RestResult<HealtCodeUserVO> getOpenResult(@RequestBody AlipayUserInfoReqDto dto) {
//        return healthCodeService.getOpenResult(dto);
//    }


    /*
    * 接受主服务发过来的设备配置信息，对缓存进行操作
    *
    * */
    @ApiOperation("接受主服务的设备配置信息")
    @PostMapping("/getSnConfig")
    public RestResult<String> getSnConfig(@RequestBody RestResult<PlaceInfoVO> rest) {
        log.info("接受主服务的设备配置信息,{}", JSON.toJSONString(rest));
        placeHealthCodeServer.getSnConfig(rest);
        return RestResponse.ok();
    }
}
