package com.yixin.ms.controller;


import com.alibaba.fastjson.JSON;
import com.yixin.ms.model.dto.IrsShortTermDTO;
import com.yixin.ms.model.dto.IrsTripCheckScheduleDTO;
import com.yixin.ms.model.dto.IrsTripSendMsgDTO;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.vo.IrsTripResultVO;
import com.yixin.ms.service.IrsTripService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


/**
 * @author lileilei
 */
@Slf4j
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "省共享行程 API")
@RestController
@RequestMapping("/irs/trip")
public class IrsTripController {
    /**
     * 服务对象
     */
    private final IrsTripService irsTripService;

    @ApiOperation("行程核验查询短信验证服务")
    @PostMapping("/sendMsg")
    public String sendMsg(@RequestBody IrsTripSendMsgDTO sendMsgDTO) {
        String body = null;
        try {
            body = irsTripService.sendMsg(sendMsgDTO);
        } catch (Exception e) {
            return JSON.toJSONString(new IrsTripResultVO<>().setCode(IrsTripResultVO.UNKNOWN_FLAG).setMsg("请求服务过于频繁，请重试"));
        }
        return body;
    }

    @ApiOperation("行程核验信息")
    @PostMapping("/checkSchedule")
    public String checkSchedule(@RequestBody IrsTripCheckScheduleDTO checkScheduleDTO) {
        String body = null;
        try {
            body = irsTripService.checkSchedule(checkScheduleDTO);
        } catch (Exception e) {
            return JSON.toJSONString(new IrsTripResultVO<>().setCode(IrsTripResultVO.UNKNOWN_FLAG).setMsg("请求服务过于频繁，请重试"));
        }
        return body;
    }

    @ApiOperation("获取省健康码接口")
    @PostMapping("/getProvinceHealthCode")
    public String getProvinceHealthCode(@RequestBody IrsShortTermDTO irsShortTermDTO) {
        String body = null;
        try {
            body = irsTripService.getProvinceHealthCode(irsShortTermDTO);
        } catch (Exception e) {
            return JSON.toJSONString(new IrsTripResultVO<>().setCode(IrsTripResultVO.UNKNOWN_FLAG).setMsg("请求服务过于频繁，请重试"));
        }
        return body;
    }

}
