package com.yixin.ms.controller;


import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.yixin.ms.boot.restful.CommonException;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.uitls.Preconditions;
import com.yixin.ms.model.dto.LinkageDto;
import com.yixin.ms.model.dto.LinkageItemDto;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.service.HealthCodeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


@Slf4j
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Api(tags = "健康码查询 API")
@RestController
@RequestMapping("/Face")
public class LinkageController {
    /**
     * 服务对象
     */
    private final HealthCodeService healthCodeService;

    @ApiOperation("联动平台接口")
    @PostMapping("/pushRecords")
    public RestResult<HealtCodeUserVO> getShortTermCertificate(@RequestBody LinkageDto dto) {
        log.info("---->联动平台接口入参:{}", JSONObject.toJSONString(dto));
        if (Preconditions.isBlank(dto) || Preconditions.isBlank(dto.getData())) {
            throw new CommonException("联动平台接口入参有误");
        }
        List<LinkageItemDto> itemDtos = dto.getData();
        LinkageItemDto itemDto = itemDtos.get(0);

        ShortTermDTO item = new ShortTermDTO();
        item.setVerifyType(itemDto.getVerifytype());
        item.setSn(dto.getSN());
        item.setIdNumber(itemDto.getIdNumber());
        item.setName(itemDto.getName());
        item.setPhone(itemDto.getPhone());
        item.setQrcode(itemDto.getQrcode());
        item.setCardIdEx(itemDto.getCardIdEx());
        //兼容老版本
        if (Preconditions.isNotBlank(itemDto.getVerifytype()) && ("0".equals(itemDto.getVerifytype()) || "1".equals(itemDto.getVerifytype()) || "2".equals(itemDto.getVerifytype()))) {
            if(StrUtil.isNotBlank(itemDto.getCardid())){item.setIdNumber(itemDto.getCardid());}
            if(StrUtil.isNotBlank(itemDto.getName())){item.setName(itemDto.getName());}
            if(StrUtil.isNotBlank(itemDto.getPhone())){item.setPhone(itemDto.getPhone());}
        }

        if (Preconditions.isNotBlank(itemDto.getVerifytype()) && ("3".equals(itemDto.getVerifytype()) || "4".equals(itemDto.getVerifytype()))) {
            if(StrUtil.isNotBlank(itemDto.getCardid())){item.setHealthCode(itemDto.getCardid());}
            if(StrUtil.isNotBlank(itemDto.getQrcode())){
                item.setHealthCode(itemDto.getQrcode());
            }
        }

        //兼容新版本二维码逻辑 必要条件：不开启三方联动
        if (Preconditions.isNotBlank(itemDto.getHhFaceVersion())) {
            if(StrUtil.isNotBlank(itemDto.getCardid()) && !IdcardUtil.isValidCard(itemDto.getCardid())){
                item.setHealthCode(itemDto.getCardid());
            }
            if(StrUtil.isNotBlank(itemDto.getQrcode())){
                item.setHealthCode(itemDto.getQrcode());
            }
        }

        item.setTaskId(StrUtil.isNotBlank(itemDto.getExtend2())? itemDto.getExtend2() : itemDto.getTaskId());
        item.setValue(StrUtil.isNotBlank(itemDto.getExtend7())? itemDto.getExtend7() : itemDto.getValue());
        item.setAction(itemDto.getAction());
        item.setPhoto(itemDto.getPhoto());
        item.setExtInfo(itemDto.getExtInfo());
        item.setHhFaceVersion(itemDto.getHhFaceVersion());
        item.setToken("hh_token_51063b27-9fd8-420d-b7eb-e5600fe0dec9");
        return healthCodeService.getShortTerm(item);
    }


}
