package com.yixin.ms.server;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.IORuntimeException;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.math.MathUtil;
import cn.hutool.core.util.*;
import cn.hutool.http.HttpException;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.yixin.ms.boot.restful.RestResponse;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.restful.Result;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.boot.uitls.ClassUtil;
import com.yixin.ms.boot.uitls.Preconditions;
import com.yixin.ms.boot.uitls.RedisUtil;
import com.yixin.ms.boot.uitls.SMSOrIdCardUtils;
import com.yixin.ms.crypto.util.Sm4Util;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.dto.ThirdPlatformDTO;
import com.yixin.ms.model.dto.ThirdPlatformRespDto;
import com.yixin.ms.model.dto.ThirdPlatformSm4DTO;
import com.yixin.ms.model.dto.ThirdPlatformSm4RespDto;
import com.yixin.ms.model.enums.*;
import com.yixin.ms.model.vo.*;
import com.yixin.ms.server.dto.QuTongIdentityAuthDto;
import com.yixin.ms.server.dto.ScenicVerifyDTO;
import com.yixin.ms.server.vo.PlaceInfoVO;
import com.yixin.ms.server.vo.QuTongIdentityVO;
import com.yixin.ms.server.vo.QuTongIdentityVO.RespCodeEnum;
import com.yixin.ms.server.vo.ScenicVerifyVO;
import com.yixin.ms.service.impl.HealthCodeServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.net.SocketTimeoutException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @description:
 * @ClassName HealthOpenReturnServer
 * @Author WXG
 * @Date 2022/10/20 17:34
 */
@Slf4j
@Service
public class HealthOpenReturnServer {

    @Autowired
    private ScenicServer scenicServer;

    @Autowired
    private QuTongCodeServer quTongCodeServer;

    @Autowired
    private PlaceHealthCodeServer placeHealthCodeServer;

    @Resource
    private UserBlackServer userBlackServer;

    //默认广告地址
    public static String defaultAdvertise = "https://tx.hz-hanghui.com:8088/yx-fyzd/file/icon/ad.png";

    //连续模式默认背景图
    String defaultABackgroundUrl = "https://tx.hz-hanghui.com:8088/yx-fyzd/file/icon/aBackground.png";
    String defaultBBackgroundUrl = "https://tx.hz-hanghui.com:8088/yx-fyzd/file/icon/bBackground.png";

    //通用模式
    private static String SHOW_PAGE_TYPE_COMMON = "common";
    //景区模式
    private static String SHOW_PAGE_TYPE_SCENIC = "scenic";
    //状态
    private static String STATUS_LOADING = "loading";
    private static String STATUS_PENDING = "pending";
    private static String STATUS_PENDING_MULTIPLE = "pendingMultiple";
    private static String STATUS_CLOSING = "closing";

    @Value("${web.address}")
    private String address;

    /**
     * 65周岁预约不限制通行
     */
    private static final int AGE = 65;


    public void healthOpenReturn(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO, ShortTermDTO shortTermDTO) {

        //新版UI 标识‼️‼️‼️
        if (Preconditions.isNotBlank(placeInfoVO)
                && Preconditions.isNotBlank(placeInfoVO.getLatestResultPage())
                && placeInfoVO.getLatestResultPage()) {
            userVO.setShowPageType("scenic");
        }

        //开门声音提示
        String openMsg = "";
        String openVoiceMsg = "";

        /**
         * 开门规则判定校验
         */
        if (placeInfoVO != null && placeInfoVO.getScenicFlag()) {
            userVO.setIsOpen(HealthOpenEnum.RELEASE.getCode());
            openMsg = "欢迎入园";
        } else {
            userVO.setIsOpen(HealthOpenEnum.RELEASE.getCode());
            openMsg = "请通行";
        }

        /*******************************以下为账号配置逻辑*******************************/
        //设备人员是否存在
        if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0)
                && Preconditions.isNotBlank(userVO.getIdNumber())
                && Preconditions.isNotBlank(shortTermDTO.getSn())
                && Preconditions.isNotBlank(placeInfoVO.getAdminId())
                && Preconditions.isNotBlank(quTongCodeServer.getAdminId())
                && !quTongCodeServer.getAdminId().contains(placeInfoVO.getAdminId())
                && (Preconditions.isBlank(placeInfoVO.getOpenThirdLinkPlatform()) || !placeInfoVO.getOpenThirdLinkPlatform())) {
            RestResult result = placeHealthCodeServer.checkIotUserPass(userVO.getIdNumber(), shortTermDTO.getSn());
            if (Preconditions.isBlank(result) || result.getCode() != RestCodeEnum.SUCCESS.getCode()) {
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                openMsg = result.getMsg();
            }
        }

        //黑名单校验
        if (Preconditions.isNotBlank(placeInfoVO.getAdminId())
                && Preconditions.isNotBlank(quTongCodeServer.getAdminId())
                && !quTongCodeServer.getAdminId().contains(placeInfoVO.getAdminId())
                && (Preconditions.isBlank(placeInfoVO.getOpenThirdLinkPlatform()) || !placeInfoVO.getOpenThirdLinkPlatform())) {
            Boolean isPassBlackValidate = userBlackServer.checkUserBlackListURL(userVO.getIdNumber(), shortTermDTO.getSn());
            placeInfoVO.setIsThroughBlackCheck(isPassBlackValidate);
            if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && !isPassBlackValidate) {
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                openMsg = "未授权，请联系工作人员";
            }
        }

        //是否是允许通行的身份证地区
        Boolean isNoAllowAreaCode = true;
        String noAllowAreaCode = placeInfoVO.getNoAllowAreaCode();
        if (StrUtil.isNotEmpty(noAllowAreaCode) && StrUtil.isNotEmpty(userVO.getIdNumber())) {
            String[] split = noAllowAreaCode.split("/");
            for (String areaCode : split) {
                if (userVO.getIdNumber().startsWith(areaCode)) {
                    isNoAllowAreaCode = false;
                    break;
                }
            }
        }
        if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0) && !isNoAllowAreaCode) {
            userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
            openMsg = HealthOpenEnum.NO_THOROUGHFARE.getDesc();
        }

        //TODO。。。身份证校验是否未满18周岁 目前未加逻辑 默认false 通行区间
        //场所类型
        Integer placeType = placeInfoVO.getPlaceType();
        String notAllowAge = placeInfoVO.getNotAllowAge();
        String notAllowAgeMsg = placeInfoVO.getNotAllowAgeMsg();
        if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0)
                && Preconditions.isNotBlank(userVO.getIdNumber())
                && Preconditions.isNotBlank(notAllowAge)
                && Preconditions.isNotBlank(placeType)
                && (placeType.intValue() == 0 || placeType.intValue() == 1)) {
            int age = IdcardUtil.getAgeByIdCard(userVO.getIdNumber());
            String[] list = notAllowAge.split("-");
            if (list.length == 2
                    && (age >= Integer.parseInt(list[0]) && age <= Integer.parseInt(list[1]))) {
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                openMsg = HealthOpenEnum.NO_THOROUGHFARE.getDesc();
                if (Preconditions.isNotBlank(notAllowAgeMsg)) {
                    openMsg = notAllowAgeMsg;
                }
            }
        }

        //TODO...处理系统配置参数 系统配置参数优先与默认参数 （通行结果，语音播报内容除外）
        convertOpenReturn(userVO, placeInfoVO);
        //TODO...衢通码 逻辑
        handleQutong(placeInfoVO, userVO, shortTermDTO, openMsg);
        /*******************************以下为三方平台逻辑*******************************/
        //TODO...联动平台三方接口 三方接口配置参数优先与系统配置参数
        handleThirdPlatform(userVO, placeInfoVO, shortTermDTO);
//        TODO...mock数据流程
//        handleThirdPlatformMock(userVO, placeInfoVO, shortTermDTO);
        /*******************************以下为通行结果逻辑*******************************/
        //通行结果转换
        extracted(userVO, placeInfoVO, openMsg);
    }

    private static void extracted(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO, String openMsg) {
        String openVoiceMsg;
        String subLine = openMsg;
        if (userVO.getIsOpen() != null
                && userVO.getIsOpen().intValue() == HealthOpenEnum.NO_THOROUGHFARE.getCode().intValue()) {//禁止通行
            //是否开启警报
            userVO.setIsAlarm(placeInfoVO.getIsAlarm());
            userVO.setVaccineIconUrl("https://tx.hz-hanghui.com:8088/yx-fyzd/file/icon/warning.png");
            userVO.setVaccineNumColor("#ed593b");
            openMsg = Preconditions.isNotBlank(placeInfoVO.getOpenVoiceMsgFail()) ? placeInfoVO.getOpenVoiceMsgFail() : (Preconditions.isNotBlank(openMsg) ? openMsg : "禁止通行");
            subLine = Preconditions.isNotBlank(placeInfoVO.getSubLineFail()) ? placeInfoVO.getSubLineFail() : (Preconditions.isNotBlank(subLine) ? subLine : (Preconditions.isNotBlank(placeInfoVO.getOpenVoiceMsgFail()) ? placeInfoVO.getOpenVoiceMsgFail() : "禁止通行"));

            userVO.setOpenTimes(0);
            userVO.setAMainIcon(Preconditions.isNotBlank(userVO.getAMainIcon()) ? userVO.getAMainIcon() : ThirdPlatformPassIconEnum.AFORBID.getUrl());
            userVO.setAMainLine(Preconditions.isNotBlank(userVO.getAMainLine()) ? userVO.getAMainLine() : openMsg);
        } else {//请通行
            openMsg = Preconditions.isNotBlank(placeInfoVO.getOpenVoiceMsgSuccess()) ? placeInfoVO.getOpenVoiceMsgSuccess() : openMsg;
            subLine = Preconditions.isNotBlank(placeInfoVO.getSubLineSuccess()) ? placeInfoVO.getSubLineSuccess() : subLine;
            userVO.setAMainIcon(Preconditions.isNotBlank(userVO.getAMainIcon()) ? userVO.getAMainIcon() : ThirdPlatformPassIconEnum.APASS.getUrl());
            userVO.setAMainLine(Preconditions.isNotBlank(userVO.getAMainLine()) ? userVO.getAMainLine() : openMsg);
        }

        //设置开门信号提示信息
        openVoiceMsg = openMsg;
        userVO.setOpenMsg(Preconditions.isNotBlank(subLine) ? subLine : openMsg);
        userVO.setOpenVoiceMsg(openVoiceMsg);
        //设置通行结果数据
        HealtCodeTypeVO codeTypeVO = userVO.getData().stream().findFirst().get();
        if (Preconditions.isNotBlank(codeTypeVO)) {
            codeTypeVO.setContent(subLine);
            userVO.setData(Arrays.asList(codeTypeVO));
        }
    }

    private void handleThirdPlatformMock(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO, ShortTermDTO shortTermDTO) {
        String thirdOpenMsg = null;
        if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0)
                && Preconditions.isNotBlank(placeInfoVO.getOpenThirdLinkPlatform())
                && placeInfoVO.getOpenThirdLinkPlatform()) {

            if (Preconditions.isNotBlank(placeInfoVO.getInterfaceAddress())) {
                try {
                    ThirdPlatformDTO dto = new ThirdPlatformDTO();
                    if (Preconditions.isNotBlank(shortTermDTO.getHealthCode())) {//二维码
                        dto.setVerifyType("QRCODE");
                        userVO.setVaccineNum("二维码认证");
                    } else {//身份证
                        dto.setVerifyType("IDCARD");
                        userVO.setVaccineNum("身份证认证");
                    }
                    dto.setSn(shortTermDTO.getSn());
                    dto.setIdNumber(userVO.getIdNumber());
                    dto.setName(userVO.getUsername());
                    dto.setPhone(shortTermDTO.getPhone());
                    dto.setQrcode(shortTermDTO.getHealthCode());
                    //20230726连续任务参数
                    dto.setTaskId(shortTermDTO.getTaskId());
                    dto.setValue(shortTermDTO.getValue());
                    ThirdPlatformSm4DTO sm4Dto = new ThirdPlatformSm4DTO();
                    long currentTimeMillis = System.currentTimeMillis();
                    sm4Dto.setReqTimestamp(currentTimeMillis);
                    sm4Dto.setSign(Sm4Util.getSign(placeInfoVO.getAppKey(), placeInfoVO.getAppSecret(), String.valueOf(currentTimeMillis)));
                    sm4Dto.setBizContent(Sm4Util.encrypt(placeInfoVO.getPrivateKey(), JSONObject.toJSONString(dto)));
                    log.info("联动平台三方接口调用加密前入参信息：{}", JSON.toJSONString(dto));

                    //TODO。mock数据
//                    ThirdPlatformRespDto mockDto = mockLoading();
//                    ThirdPlatformRespDto mockDto = mockPending();
                    ThirdPlatformRespDto mockDto = mockPendingMultiple();
//                    ThirdPlatformRespDto mockDto = mockClosing();
                    convertThirdOpenReturn(userVO, mockDto, placeInfoVO);
                    return;
                } catch (Exception e) {
                    userVO.setOpenTimes(0);
                    log.error("联动平台三方接口响应异常：{}", e.getMessage());
                    e.printStackTrace();
                    thirdOpenMsg = "联动平台三方接口响应异常，禁止通行";
                }
            }
            throw new ServiceException(thirdOpenMsg);
        }

    }

    private ThirdPlatformRespDto mockLoading() {
        ThirdPlatformRespDto thirdPlatformRespDto = new ThirdPlatformRespDto();
        thirdPlatformRespDto.setShowPageType(SHOW_PAGE_TYPE_SCENIC);
        thirdPlatformRespDto.setStatus(STATUS_LOADING);
        thirdPlatformRespDto.setTaskId(UUID.randomUUID().toString());
//            thirdPlatformRespDto.setDBackgroundUrl(thirdPlatformRespDto.getDBackgroundUrl());
        int i = RandomUtil.randomInt(1, 10);
        thirdPlatformRespDto.setDMainLine("处理中..." + i + "（/2）");
        thirdPlatformRespDto.setDelay(500);
        log.info("========" + i);
        if (i == 8) {
            thirdPlatformRespDto.setEnd(Boolean.TRUE);
        } else {
            thirdPlatformRespDto.setEnd(Boolean.FALSE);
        }
        return thirdPlatformRespDto;
    }


    private ThirdPlatformRespDto mockPending() {
        ThirdPlatformRespDto thirdPlatformRespDto = new ThirdPlatformRespDto();
        thirdPlatformRespDto.setShowPageType(SHOW_PAGE_TYPE_SCENIC);
        thirdPlatformRespDto.setStatus(STATUS_PENDING);

        thirdPlatformRespDto.setBBackgroundUrl(null);
        thirdPlatformRespDto.setTaskId(UUID.randomUUID().toString());
        //设置默认图
        thirdPlatformRespDto.setBMainIcon(ThirdPlatformPassIconEnum.BWARNING.getIconCode());
        thirdPlatformRespDto.setBMainLine("等待二次核验");
        thirdPlatformRespDto.setBSubLine("新金华人免费预约票");
        List<ChooseVO> bChooseList = new ArrayList<>();
        ChooseVO cv = new ChooseVO();
        cv.setDeep("false");
        cv.setLabel("确认核销");
        cv.setValue("1");
        ChooseVO cv2 = new ChooseVO();
        cv2.setDeep("true");
        cv2.setLabel("取消");
        cv2.setValue("2");
        ChooseVO cv3 = new ChooseVO();
        cv3.setDeep("false");
        cv3.setLabel("选项三");
        cv3.setValue("3");
        bChooseList.add(cv);
        bChooseList.add(cv2);
        bChooseList.add(cv3);
        thirdPlatformRespDto.setBChoiseList(bChooseList);
        return thirdPlatformRespDto;
    }

    private ThirdPlatformRespDto mockPendingMultiple() {
        ThirdPlatformRespDto thirdPlatformRespDto = new ThirdPlatformRespDto();
        thirdPlatformRespDto.setShowPageType(SHOW_PAGE_TYPE_SCENIC);
        thirdPlatformRespDto.setStatus(STATUS_PENDING_MULTIPLE);
        thirdPlatformRespDto.setCButtonText("确认核销");
        thirdPlatformRespDto.setCButtonSubText("已勾选$count张票");
        thirdPlatformRespDto.setTaskId("-1");

//        thirdPlatformRespDto.setCBackgroundUrl(thirdPlatformRespDto.getCBackgroundUrl());
        MultipleChooseVO vo1 = new MultipleChooseVO();
        MultipleChooseItemVO item1 = new MultipleChooseItemVO();
        item1.setLabel("成人票-1");
        item1.setSelected(Boolean.TRUE);
        item1.setValue("CRP202308300001");
        MultipleChooseItemVO item2 = new MultipleChooseItemVO();
        item2.setLabel("成人票-2");
        item2.setSelected(Boolean.FALSE);
        item2.setValue("CRP202308300002");
        MultipleChooseItemVO item3 = new MultipleChooseItemVO();
        item3.setLabel("成人票-3");
        item3.setSelected(Boolean.FALSE);
        item3.setValue("CRP202308300003");
        MultipleChooseItemVO item4 = new MultipleChooseItemVO();
        item4.setLabel("成人票-4");
        item4.setSelected(Boolean.FALSE);
        item4.setValue("CRP202308300004");
        vo1.setList(Arrays.asList(item1, item2, item2, item4));
        vo1.setTitle("吴依依");
        vo1.setSubTitle("SFZ202308300002");

        MultipleChooseVO vo2 = new MultipleChooseVO();
        MultipleChooseItemVO vo2item1 = new MultipleChooseItemVO();
        vo2item1.setLabel("儿童票-1");
        vo2item1.setSelected(Boolean.FALSE);
        vo2item1.setValue("ETP202308300001");
        MultipleChooseItemVO vo2item2 = new MultipleChooseItemVO();
        vo2item2.setLabel("儿童票-2");
        vo2item2.setSelected(Boolean.TRUE);
        vo2item2.setValue("ETP202308300002");
        MultipleChooseItemVO vo2item3 = new MultipleChooseItemVO();
        vo2item3.setLabel("儿童票-3");
        vo2item3.setSelected(Boolean.FALSE);
        vo2item3.setValue("ETP202308300003");
        MultipleChooseItemVO vo2item4 = new MultipleChooseItemVO();
        vo2item4.setLabel("儿童票-4");
        vo2item4.setSelected(Boolean.FALSE);
        vo2item4.setValue("ETP202308300004");
        vo2.setList(Arrays.asList(vo2item1, vo2item2, vo2item3, vo2item4));
        vo2.setTitle("吴邪");
        vo2.setSubTitle("ETSFZ202308300004");


        MultipleChooseVO vo3 = new MultipleChooseVO();
        MultipleChooseItemVO vo3item1 = new MultipleChooseItemVO();
        vo3item1.setLabel("联票V1-1");
        vo3item1.setSelected(Boolean.FALSE);
        vo3item1.setValue("LP202308300001");
        MultipleChooseItemVO vo3item2 = new MultipleChooseItemVO();
        vo3item2.setLabel("联票V1-2");
        vo3item2.setSelected(Boolean.TRUE);
        vo3item2.setValue("L202308300002");
        MultipleChooseItemVO vo3item3 = new MultipleChooseItemVO();
        vo3item3.setLabel("联票V1-3");
        vo3item3.setSelected(Boolean.FALSE);
        vo3item3.setValue("LP202308300003");
        vo3.setList(Arrays.asList(vo3item1, vo3item2, vo3item3));
        vo3.setTitle("联票V1");
        vo3.setSubTitle("LPSFZ202308300004");

        MultipleChooseVO vo4 = new MultipleChooseVO();
        MultipleChooseItemVO vo4item1 = new MultipleChooseItemVO();
        vo4item1.setLabel("联票V2-1");
        vo4item1.setSelected(Boolean.FALSE);
        vo4item1.setValue("VLP202308300001");
        MultipleChooseItemVO vo4item2 = new MultipleChooseItemVO();
        vo4item2.setLabel("联票V2-2");
        vo4item2.setSelected(Boolean.FALSE);
        vo4item2.setValue("VLP202308300002");
        MultipleChooseItemVO vo4item3 = new MultipleChooseItemVO();
        vo4item3.setLabel("联票V2-3");
        vo4item3.setSelected(Boolean.TRUE);
        vo4item3.setValue("VLPLP202308300003");
        vo4.setList(Arrays.asList(vo4item1, vo4item2, vo4item3));
        vo4.setTitle("联票V2");
        vo4.setSubTitle("VLPSFZ202308300004");

        thirdPlatformRespDto.setCChoiseList(Arrays.asList(vo1, vo2, vo3, vo4));
        return thirdPlatformRespDto;
    }

    private ThirdPlatformRespDto mockClosing() {
        ThirdPlatformRespDto thirdPlatformRespDto = new ThirdPlatformRespDto();
        thirdPlatformRespDto.setShowPageType(SHOW_PAGE_TYPE_SCENIC);
        thirdPlatformRespDto.setStatus(STATUS_CLOSING);
        thirdPlatformRespDto.setABackgroundUrl(null);
        //头部
        if (Preconditions.isNotBlank(thirdPlatformRespDto)
                && Preconditions.isNotBlank(thirdPlatformRespDto.getAMainLine())) {
            thirdPlatformRespDto.setAMainLine(thirdPlatformRespDto.getAMainLine());
        }

        thirdPlatformRespDto.setAMainIcon(ThirdPlatformPassIconEnum.APASS.getIconCode());
        thirdPlatformRespDto.setAMainLine("检票成功");
        thirdPlatformRespDto.setOpenVoiceMsg("检票成功");

        thirdPlatformRespDto.setASubLineFirst("曾**");
        thirdPlatformRespDto.setASubLineSecond("3308**********0212");
        thirdPlatformRespDto.setALineOneLabel("票名");
        thirdPlatformRespDto.setALineOneValue("年卡（季卡）");
        thirdPlatformRespDto.setALineTwoLabel("有效期");
        thirdPlatformRespDto.setALineTwoValue("2023-08-07");
        thirdPlatformRespDto.setALineThrLabel("年卡照片");
        thirdPlatformRespDto.setALineThrValue(ThirdPlatformPassIconEnum.APASS.getUrl());
        //底部
        thirdPlatformRespDto.setQrcodeText("QrcodeText");
        thirdPlatformRespDto.setAdvertise("Advertise");
        //开门信号格式
        thirdPlatformRespDto.setIsOpen(0);
        thirdPlatformRespDto.setSignalType("default");
        thirdPlatformRespDto.setOpenTimes(1);
        return thirdPlatformRespDto;
    }

    /**
     * 三方接入平台
     *
     * @param userVO
     * @param placeInfoVO
     * @param shortTermDTO
     */
    private void handleThirdPlatform(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO, ShortTermDTO shortTermDTO) {
        String thirdOpenMsg = null;
        if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0)
                && Preconditions.isNotBlank(placeInfoVO.getOpenThirdLinkPlatform())
                && placeInfoVO.getOpenThirdLinkPlatform()
                && Preconditions.isNotBlank(placeInfoVO.getAppKey())
                && Preconditions.isNotBlank(placeInfoVO.getAppSecret())
                && Preconditions.isNotBlank(placeInfoVO.getPrivateKey())) {
            if (Preconditions.isNotBlank(placeInfoVO.getInterfaceAddress())) {
                try {

                    ThirdPlatformDTO dto = new ThirdPlatformDTO();
                    VerifyTypeEnum verifyTypeEnum = VerifyTypeEnum.getByCode(shortTermDTO.getVerifyType());
                    dto.setVerifyType(ObjectUtil.isNull(verifyTypeEnum) ? shortTermDTO.getVerifyType() : verifyTypeEnum.getText());
                    if (ObjectUtil.isNotNull(verifyTypeEnum)) {
                        userVO.setVaccineNum(verifyTypeEnum.getDesc());
                    } else {
                        dto.setVerifyType(VerifyTypeEnum.IDCARD.getText());
                    }
                    if (Preconditions.isBlank(shortTermDTO.getVerifyType())
                            && "3".equals(shortTermDTO.getVerifyType())) {//二维码 兼容1831
                        dto.setVerifyType("QRCODE");
                        userVO.setVaccineNum("二维码认证");
                    }
                    dto.setSn(shortTermDTO.getSn());
                    dto.setIdNumber(userVO.getIdNumber());
                    dto.setName(userVO.getUsername());
                    dto.setPhone(shortTermDTO.getPhone());
                    dto.setQrcode(shortTermDTO.getHealthCode());
                    //20230726连续任务参数
                    dto.setTaskId(shortTermDTO.getTaskId());
                    dto.setValue(shortTermDTO.getValue());

                    ThirdPlatformSm4DTO sm4Dto = new ThirdPlatformSm4DTO();
                    long currentTimeMillis = System.currentTimeMillis();
                    sm4Dto.setReqTimestamp(currentTimeMillis);
                    sm4Dto.setSign(Sm4Util.getSign(placeInfoVO.getAppKey(), placeInfoVO.getAppSecret(), String.valueOf(currentTimeMillis)));
                    sm4Dto.setBizContent(Sm4Util.encrypt(placeInfoVO.getPrivateKey(), JSONObject.toJSONString(dto)));
                    log.info("票务核验接口调用加密前入参信息：{}", JSON.toJSONString(dto));
                    long startTime = System.currentTimeMillis();
                    String body = HttpUtil.post(placeInfoVO.getInterfaceAddress(), JSONObject.toJSONString(sm4Dto), 4000);
                    log.info("----->票务核验接口响应耗时:{}", System.currentTimeMillis() - startTime);
                    RestResult result = JSON.parseObject(body, RestResult.class);
                    log.info("票务核验接口响应原始信息：{}", JSON.toJSONString(result));
                    if (Preconditions.isNotBlank(result)
                            && result.getCode() == RestCodeEnum.SUCCESS.getCode()
                            && Preconditions.isNotBlank(result.getData())) {
                        ThirdPlatformSm4RespDto thirdPlatformSm4RespDto = JSON.parseObject(JSON.toJSONString(result.getData()), ThirdPlatformSm4RespDto.class);
                        //验证签名
                        Boolean checkSign = Sm4Util.checkSign(placeInfoVO.getAppKey(), placeInfoVO.getAppSecret(), thirdPlatformSm4RespDto.getSign(), String.valueOf(thirdPlatformSm4RespDto.getReqTimestamp()));
                        if (checkSign) {
                            String decrypt = Sm4Util.decrypt(placeInfoVO.getPrivateKey(), thirdPlatformSm4RespDto.getBizContent());
                            log.info("票务核验接口响应内容解密：{}", decrypt);
                            ThirdPlatformRespDto thirdPlatformRespDto = JSON.parseObject(decrypt, ThirdPlatformRespDto.class);
                            convertThirdOpenReturn(userVO, thirdPlatformRespDto, placeInfoVO);
                            return;
                        } else {
                            thirdOpenMsg = "票务核验接口返回签名错误";
                        }
                    } else {
                        thirdOpenMsg = Preconditions.isNotBlank(result.getMsg()) ? result.getMsg() : null;
                    }
                } catch (ServiceException e) {
                    log.error("--->ServiceException票务核验接口响应异常：{}", e.getMessage());
                    e.printStackTrace();
                    thirdOpenMsg = StrUtil.isBlank(e.getMessage()) ? "票务核验接口响应异常" : e.getMessage();
                } catch (IORuntimeException e) {
                    log.error("--->IORuntimeException票务核验接口响应异常：{}", e.getMessage());
                    e.printStackTrace();
                    thirdOpenMsg = "票务核验超时";
                } catch (HttpException e) {
                    log.error("--->HttpException票务核验接口响应异常：{}", e.getMessage());
                    e.printStackTrace();
                    thirdOpenMsg = "票务核验超时";
                } catch (Exception e) {
                    log.error("--->Exception票务核验接口响应异常：{}", e.getMessage());
                    e.printStackTrace();
                    thirdOpenMsg = "票务核验接口响应异常";
                    if (Preconditions.isNotBlank(e.getMessage())
                            && e.getMessage().contains("Read timed out")) {
                        thirdOpenMsg = "票务核验超时";
                    }
                }
            } else {
                thirdOpenMsg = "票务核验接口未配置\n请联系管理员";
            }
            throw new ServiceException(Preconditions.isNotBlank(thirdOpenMsg) ? thirdOpenMsg : (Preconditions.isNotBlank(placeInfoVO.getOpenVoiceMsgFail()) ? placeInfoVO.getOpenVoiceMsgFail() : "禁止通行"));
        }
    }

    public static void main(String[] args) {
        String decrypt = "{\"taskId\":\"akdjflsdkfjald\",\"delay\":5000,\"end\":false,\"isOpen\":1,\"openTimes\":1,\"openVoiceMsg\":\"请等待\",\"signalNo\":\"\",\"signalType\":\"default\",\"name\":\"尚义鹏\",\"idNumber\":\"410922199611150054\",\"photo\":\"\",\"status\":\"loading\"}\n";
        ThirdPlatformRespDto thirdPlatformRespDto = JSON.parseObject(decrypt, ThirdPlatformRespDto.class);
        log.info("输出{}", JSON.toJSONString(thirdPlatformRespDto));
    }

    /*开始
     * 新版本下直接请求三方
     * */
    public RestResult<HealtCodeUserVO> handleThirdPlatform(ShortTermDTO shortTermDTO, PlaceInfoVO placeInfoVO) {
        //前置的数据校验
        this.checkThirdPlatform(placeInfoVO);
        //1.将数据转换成传递给三方的参数对象
        ThirdPlatformDTO dto = this.shortTermDtoConvetThirdPlatformDto(shortTermDTO);
        //2.对参数进行加密
        ThirdPlatformSm4DTO sm4Dto = this.encrypThirdPlatformDTO(dto, placeInfoVO);
        //3.调用三方
        ThirdPlatformRespDto respDto = this.invokeThirdPlatform(shortTermDTO.getSn(), sm4Dto, placeInfoVO);
        //4.整合返回数据
        return this.convertThirdOpenReturn(shortTermDTO, respDto, placeInfoVO);
    }

    private void checkThirdPlatform(PlaceInfoVO placeInfoVO) {
        ClassUtil.throwBusinessException(StrUtil.isBlank(placeInfoVO.getInterfaceAddress()), "联动平台三方接口未配置\n请联系管理员");
        ClassUtil.throwBusinessException(StrUtil.isBlank(placeInfoVO.getAppKey()), "appKey未配置\n请联系管理员");
        ClassUtil.throwBusinessException(StrUtil.isBlank(placeInfoVO.getAppSecret()), "appSecret未配置\n请联系管理员");
        ClassUtil.throwBusinessException(StrUtil.isBlank(placeInfoVO.getPrivateKey()), "privateKey未配置\n请联系管理员");

    }

    private RestResult<HealtCodeUserVO> convertThirdOpenReturn(ShortTermDTO shortTermDTO, ThirdPlatformRespDto thirdPlatformRespDto, PlaceInfoVO placeInfoVO) {
        HealtCodeUserVO userVO = BeanUtil.copyProperties(thirdPlatformRespDto, HealtCodeUserVO.class);
//        HealthCodeServiceImpl.initHealthUserVo(shortTermDTO, placeInfoVO);
//        convertThirdOpenReturn(userVO, thirdPlatformRespDto, placeInfoVO);
//        String openMsg = "请通行";
//        extracted(userVO, placeInfoVO, openMsg);
        return RestResponse.ok(userVO);
    }


    private ThirdPlatformRespDto invokeThirdPlatform(String sn, ThirdPlatformSm4DTO sm4Dto, PlaceInfoVO placeInfoVO) {
        try {
            Long timeStart = DateUtil.current();
            RestResult<ThirdPlatformSm4RespDto> request = HttpRequest.post(placeInfoVO.getInterfaceAddress())
                    .body(JSONObject.toJSONString(sm4Dto)).timeout(4000).thenFunction(val -> {
                        String body = val.body();
                        log.info("设备号:{},请求三方返回打印:{}", sn, body);
                        return JSON.parseObject(body, new TypeReference<RestResult<ThirdPlatformSm4RespDto>>() {
                        });
                    });
            Long timeEnd = DateUtil.current();
            log.info("设备号:{},请求三方耗时:{}", sn, timeEnd - timeStart);
            ClassUtil.throwBusinessException(
                    request.getCode() != RestCodeEnum.SUCCESS.getCode() ||
                            ObjectUtil.isEmpty(request.getData()),
                    StrUtil.isNotBlank(request.getMsg()) ? request.getMsg() : null);
            ThirdPlatformSm4RespDto thirdPlatformSm4RespDto = request.getData();
            //校验签名
            Boolean checkSign = Sm4Util.checkSign(placeInfoVO.getAppKey(), placeInfoVO.getAppSecret(), thirdPlatformSm4RespDto.getSign(), String.valueOf(thirdPlatformSm4RespDto.getReqTimestamp()));
            ClassUtil.throwBusinessException(!checkSign, "联动平台三方接口返回签名错误");
            //解密数据
            String decrypt = Sm4Util.decrypt(placeInfoVO.getPrivateKey(), thirdPlatformSm4RespDto.getBizContent());


//            HttpResponse execute = HttpRequest.post(placeInfoVO.getInterfaceAddress())
//                    .body(JSONObject.toJSONString(sm4Dto)).timeout(5000).execute();
//            String decrypt  =execute.body();
//
//            String decrypt = "{\"aBackgroundUrl\":\"\",\"aLineOneLabel\":\"票名\",\"aLineOneValue\":\"南昌八一起义纪念馆\",\"aLineThrLabel\":\"\",\"aLineThrValue\":\"https://env-00jx45j4uiq9.normal.cloudstatic.cn/byg/images/woman.png\",\"aLineTwoLabel\":\"\",\"aLineTwoValue\":\"\",\"aMainIcon\":1,\"aMainLine\":\"检票成功\",\"aSubLineFirst\":\"毛康华\",\"aSublineSecond\":\"4325**********5706\",\"advertise\":\"\",\"bBackgroundUrl\":\"\",\"bMainIcon\":1,\"bMainLine\":\"\",\"bSubLine\":\"\",\"cBackgroundUrl\":\"\",\"dBackgroundUrl\":\"\",\"dMainLine\":\"\",\"delay\":500,\"end\":true,\"isOpen\":0,\"mainLine\":\"南昌八一起义纪念馆\",\"mainLineLeft\":\"\",\"mainLineRight\":\"\",\"openTimes\":1,\"openVoiceMsg\":\"请通行\",\"qrcodeText\":\"\",\"showPageType\":\"scenic\",\"signalNo\":\"02 00 00 09 00 47 01 00 1E FF FF FF 00 00 AE 03\",\"signalType\":\"default\",\"status\":\"closing\",\"taskId\":\"\",\"subLine\":\"\",\"subLineLeft\":\"\",\"subLineRight\":\"\",\"countDown\":0}";

            log.info("设备号:{},票务核验接口响应内容解密:{}", sn, decrypt);
            return JSON.parseObject(decrypt, ThirdPlatformRespDto.class);
        } catch (ServiceException e) {
            throw new ServiceException(StrUtil.isBlank(e.getMessage()) ? "票务核验接口响应异常" : e.getMessage());
        } catch (IORuntimeException e) {
            throw new ServiceException("票务核验超时");
        } catch (HttpException e) {
            throw new ServiceException("票务核验超时");
        } catch (Exception e) {
            log.error("设备号：" + sn + "票务核验接口响应异常", e);
            if (Preconditions.isNotBlank(e.getMessage())
                    && e.getMessage().contains("Read timed out")) {
                throw new ServiceException("票务核验超时");
            }
            throw new ServiceException("票务核验接口响应异常");
        }
    }


    private ThirdPlatformSm4DTO encrypThirdPlatformDTO(ThirdPlatformDTO dto, PlaceInfoVO placeInfoVO) {
        log.info("联动平台三方接口调用加密前入参信息：{}", JSON.toJSONString(dto));
        ThirdPlatformSm4DTO sm4Dto = new ThirdPlatformSm4DTO();
        long currentTimeMillis = System.currentTimeMillis();
        sm4Dto.setReqTimestamp(currentTimeMillis);
        sm4Dto.setSign(Sm4Util.getSign(placeInfoVO.getAppKey(), placeInfoVO.getAppSecret(), String.valueOf(currentTimeMillis)));
        sm4Dto.setBizContent(Sm4Util.encrypt(placeInfoVO.getPrivateKey(), JSONObject.toJSONString(dto)));
        return sm4Dto;

    }

    private ThirdPlatformDTO shortTermDtoConvetThirdPlatformDto(ShortTermDTO shortTermDTO) {
        ThirdPlatformDTO dto = new ThirdPlatformDTO();
        VerifyTypeEnum verifyTypeEnum = VerifyTypeEnum.getByCode(shortTermDTO.getVerifyType());
        VerifyTypeEnum text = VerifyTypeEnum.getByText(shortTermDTO.getVerifyType());
        dto.setVerifyType(ObjectUtil.isNull(verifyTypeEnum) ? text.getText() : verifyTypeEnum.getText());

        dto.setSn(shortTermDTO.getSn());
        dto.setIdNumber(shortTermDTO.getIdNumber());
        dto.setName(shortTermDTO.getName());
        dto.setPhone(shortTermDTO.getPhone());
        dto.setQrcode(shortTermDTO.getQrcode());
        dto.setTaskId(shortTermDTO.getTaskId());
        dto.setValue(shortTermDTO.getValue());
        dto.setAction(shortTermDTO.getAction());
        dto.setPhoto(shortTermDTO.getPhoto());
        dto.setExtInfo(shortTermDTO.getExtInfo());
        return dto;
    }
    /*结束
     * 新版本下直接请求三方
     * */

    //处理系统配置参数
    private void convertOpenReturn(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO) {
        //头部
        if (Preconditions.isNotBlank(placeInfoVO)
                && Preconditions.isNotBlank(placeInfoVO.getTopTitle())) {
            userVO.setTitle(placeInfoVO.getTopTitle());
        }
        //中下左部
        HealtCodeNucleinVO v = userVO.getNuclein().get(0);
        v.setTitle(Preconditions.isNotBlank(placeInfoVO.getMainLineLeft()) ? placeInfoVO.getMainLineLeft() : v.getTitle());
        v.setContent(Preconditions.isNotBlank(placeInfoVO.getSubLineLeft()) ? placeInfoVO.getSubLineLeft() : v.getContent());
        userVO.setNuclein(Arrays.asList(v));
        //中下右部
        userVO.setVaccineIcoTitle(Preconditions.isNotBlank(placeInfoVO.getMainLineRight()) ? placeInfoVO.getMainLineRight() : userVO.getVaccineIcoTitle());
        userVO.setVaccineNum(Preconditions.isNotBlank(placeInfoVO.getSubLineRight()) ? placeInfoVO.getSubLineRight() : userVO.getVaccineNum());
        //底部
        userVO.setShowQrcode(placeInfoVO.getShowQrcode());
        userVO.setQrcodeText(placeInfoVO.getShowQrcode() ? placeInfoVO.getQrcodeText() : StrUtil.EMPTY);
        userVO.setAdvertise((!placeInfoVO.getShowQrcode() && Preconditions.isBlank(placeInfoVO.getAdvertise())) ? defaultAdvertise : placeInfoVO.getAdvertise());
    }

    //处理联动平台三方接口返回参数
    private void convertThirdOpenReturn(HealtCodeUserVO userVO, ThirdPlatformRespDto thirdPlatformRespDto, PlaceInfoVO placeInfoVO) {

        if (Preconditions.isNotBlank(thirdPlatformRespDto.getShowPageType())
                && !SHOW_PAGE_TYPE_SCENIC.equals(thirdPlatformRespDto.getShowPageType())
                && !SHOW_PAGE_TYPE_COMMON.equals(thirdPlatformRespDto.getShowPageType())) {
            throw new ServiceException("联动平台三方返回内容有误");
        }

        //单次模式 通用模版
        if (Preconditions.isBlank(thirdPlatformRespDto.getShowPageType()) || SHOW_PAGE_TYPE_COMMON.equals(thirdPlatformRespDto.getShowPageType())) {
            convertCommonThirdOpenReturn(userVO, thirdPlatformRespDto, placeInfoVO);
            return;
        }

        //判断是否是景区连续模式 景区模版
        if (Preconditions.isNotBlank(thirdPlatformRespDto.getStatus())
                && Preconditions.isNotBlank(thirdPlatformRespDto.getShowPageType())
                && SHOW_PAGE_TYPE_SCENIC.equals(thirdPlatformRespDto.getShowPageType())) {
            convertScenicThirdOpenReturn(userVO, thirdPlatformRespDto, placeInfoVO);
            return;
        }

    }

    private void convertScenicThirdOpenReturn(HealtCodeUserVO userVO, ThirdPlatformRespDto thirdPlatformRespDto, PlaceInfoVO placeInfoVO) {
        userVO.setMainLineLeft(placeInfoVO.getMainLineLeft());
        userVO.setMainLineRight(placeInfoVO.getMainLineRight());
        userVO.setShowPageType(thirdPlatformRespDto.getShowPageType());
        userVO.setStatus(thirdPlatformRespDto.getStatus());
        userVO.setTaskId(thirdPlatformRespDto.getTaskId());

        if (Preconditions.isNotBlank(thirdPlatformRespDto.getEnd())) {
            userVO.setEnd(thirdPlatformRespDto.getEnd());
        }

        if (Preconditions.isNotBlank(thirdPlatformRespDto.getDelay())) {
            userVO.setDelay(thirdPlatformRespDto.getDelay());
        }

        if (Preconditions.isNotBlank(thirdPlatformRespDto.getIsOpen())) {
            userVO.setIsOpen(thirdPlatformRespDto.getIsOpen());
        }
        if (userVO.getIsOpen().intValue() == 1) {
            userVO.setOpenTimes(0);
        }

        //loading状态
        if (Preconditions.isNotBlank(thirdPlatformRespDto.getStatus()) && STATUS_LOADING.equals(thirdPlatformRespDto.getStatus())) {
            userVO.setDBackgroundUrl(thirdPlatformRespDto.getDBackgroundUrl());
            userVO.setDMainLine(thirdPlatformRespDto.getDMainLine());
            placeInfoVO.setOpenVoiceMsgSuccess(Preconditions.isNotBlank(thirdPlatformRespDto.getOpenVoiceMsg()) ? thirdPlatformRespDto.getOpenVoiceMsg() : placeInfoVO.getOpenVoiceMsgSuccess());
            placeInfoVO.setOpenVoiceMsgFail(Preconditions.isNotBlank(thirdPlatformRespDto.getOpenVoiceMsg()) ? thirdPlatformRespDto.getOpenVoiceMsg() : placeInfoVO.getOpenVoiceMsgFail());
            //开门信号格式
            if (Preconditions.isNotBlank(thirdPlatformRespDto.getSignalType())) {
                userVO.setUseSerialPort(thirdPlatformRespDto.getSignalType());
            }
            if (Preconditions.isNotBlank(thirdPlatformRespDto.getOpenTimes())) {
                userVO.setOpenTimes(thirdPlatformRespDto.getOpenTimes());
            }
            if (Preconditions.isNotBlank(thirdPlatformRespDto.getSignalNo())) {
                userVO.setSignalNo(thirdPlatformRespDto.getSignalNo());
            }

            return;
        }

        //pending状态
        if (Preconditions.isNotBlank(thirdPlatformRespDto.getStatus()) && STATUS_PENDING.equals(thirdPlatformRespDto.getStatus())) {
            if (Preconditions.isBlank(thirdPlatformRespDto.getTaskId()) || Preconditions.isBlank(thirdPlatformRespDto.getBChoiseList())) {
                throw new ServiceException("PENDING状态下三方平台接口响应参数有误");
            }

            userVO.setBBackgroundUrl(thirdPlatformRespDto.getBBackgroundUrl());
            //设置默认图
            String icon = ThirdPlatformPassIconEnum.iconOf(thirdPlatformRespDto.getBMainIcon(), Boolean.FALSE);
            userVO.setBMainIcon(Preconditions.isBlank(icon) ? null : icon);
            userVO.setBSubLine(thirdPlatformRespDto.getBSubLine());
            userVO.setBMainLine(thirdPlatformRespDto.getBMainLine());
            placeInfoVO.setOpenVoiceMsgSuccess(Preconditions.isNotBlank(thirdPlatformRespDto.getOpenVoiceMsg()) ? thirdPlatformRespDto.getOpenVoiceMsg() : placeInfoVO.getOpenVoiceMsgSuccess());
            placeInfoVO.setOpenVoiceMsgFail(Preconditions.isNotBlank(thirdPlatformRespDto.getOpenVoiceMsg()) ? thirdPlatformRespDto.getOpenVoiceMsg() : placeInfoVO.getOpenVoiceMsgFail());
            userVO.setBChoiseList(thirdPlatformRespDto.getBChoiseList());
            return;
        }

        //pendingMultiple状态
        if (Preconditions.isNotBlank(thirdPlatformRespDto.getStatus()) && STATUS_PENDING_MULTIPLE.equals(thirdPlatformRespDto.getStatus())) {
            if (Preconditions.isBlank(thirdPlatformRespDto.getTaskId()) || Preconditions.isBlank(thirdPlatformRespDto.getCChoiseList())) {
                throw new ServiceException("PENDING_MULTIPLE状态下三方平台接口响应参数有误");
            }
            if (Preconditions.isNotBlank(thirdPlatformRespDto.getCButtonText())) {
                userVO.setCButtonText(thirdPlatformRespDto.getCButtonText());
            }
            if (Preconditions.isNotBlank(thirdPlatformRespDto.getCButtonSubText())) {
                userVO.setCButtonSubText(thirdPlatformRespDto.getCButtonSubText());
            }
            userVO.setCBackgroundUrl(thirdPlatformRespDto.getCBackgroundUrl());
            placeInfoVO.setOpenVoiceMsgSuccess(Preconditions.isNotBlank(thirdPlatformRespDto.getOpenVoiceMsg()) ? thirdPlatformRespDto.getOpenVoiceMsg() : placeInfoVO.getOpenVoiceMsgSuccess());
            placeInfoVO.setOpenVoiceMsgFail(Preconditions.isNotBlank(thirdPlatformRespDto.getOpenVoiceMsg()) ? thirdPlatformRespDto.getOpenVoiceMsg() : placeInfoVO.getOpenVoiceMsgFail());
            userVO.setCChoiseList(thirdPlatformRespDto.getCChoiseList());
            return;
        }

        //closing状态
        if (Preconditions.isNotBlank(thirdPlatformRespDto.getStatus()) && thirdPlatformRespDto.getStatus().contains(STATUS_CLOSING)) {
            userVO.setABackgroundUrl(thirdPlatformRespDto.getABackgroundUrl());
            //头部
            if (Preconditions.isNotBlank(thirdPlatformRespDto)
                    && Preconditions.isNotBlank(thirdPlatformRespDto.getAMainLine())) {
                userVO.setAMainLine(thirdPlatformRespDto.getAMainLine());
            }
            String icon = ThirdPlatformPassIconEnum.iconOf(thirdPlatformRespDto.getAMainIcon(), Boolean.TRUE);
            userVO.setAMainIcon(Preconditions.isBlank(icon) ? null : icon);
            placeInfoVO.setOpenVoiceMsgSuccess(Preconditions.isNotBlank(thirdPlatformRespDto.getOpenVoiceMsg()) ? thirdPlatformRespDto.getOpenVoiceMsg() : placeInfoVO.getOpenVoiceMsgSuccess());
            placeInfoVO.setOpenVoiceMsgFail(Preconditions.isNotBlank(thirdPlatformRespDto.getOpenVoiceMsg()) ? thirdPlatformRespDto.getOpenVoiceMsg() : placeInfoVO.getOpenVoiceMsgFail());
            //通行结果
            if (Preconditions.isNotBlank(thirdPlatformRespDto.getAMainIcon())
                    && thirdPlatformRespDto.getAMainIcon().intValue() == ThirdPlatformPassIconEnum.APASS.getIconCode()) {
                userVO.setAMainLine(Preconditions.isNotBlank(thirdPlatformRespDto.getAMainLine()) ? thirdPlatformRespDto.getAMainLine() : placeInfoVO.getSubLineSuccess());
            } else {
                userVO.setAMainLine(Preconditions.isNotBlank(thirdPlatformRespDto.getAMainLine()) ? thirdPlatformRespDto.getAMainLine() : placeInfoVO.getSubLineFail());
            }
            userVO.setASubLineFirst(thirdPlatformRespDto.getASubLineFirst());
            userVO.setASubLineSecond(thirdPlatformRespDto.getASubLineSecond());
            try {
                if (SMSOrIdCardUtils.isIdCard(thirdPlatformRespDto.getASubLineSecond())) {
                    userVO.setShowIdNumber(thirdPlatformRespDto.getASubLineSecond());
                    userVO.setASubLineSecond(DesensitizedUtil.idCardNum(thirdPlatformRespDto.getASubLineSecond(), 4, 4));
                }
            } catch (Exception e) {
                log.error("--->身份证校验有误");
            }
            userVO.setALineOneLabel(thirdPlatformRespDto.getALineOneLabel());
            userVO.setALineOneValue(thirdPlatformRespDto.getALineOneValue());
            userVO.setALineTwoLabel(thirdPlatformRespDto.getALineTwoLabel());
            userVO.setALineTwoValue(thirdPlatformRespDto.getALineTwoValue());
            userVO.setALineThrLabel(thirdPlatformRespDto.getALineThrLabel());
            //图片转换
            String aLineThrValue = thirdPlatformRespDto.getALineThrValue();
            userVO.setALineThrValue(aLineThrValue);
            //底部
            userVO.setQrcodeText(Preconditions.isNotBlank(thirdPlatformRespDto.getQrcodeText()) ? thirdPlatformRespDto.getQrcodeText() : userVO.getQrcodeText());
            userVO.setAdvertise(Preconditions.isNotBlank(thirdPlatformRespDto.getAdvertise()) ? thirdPlatformRespDto.getAdvertise() : userVO.getAdvertise());
            userVO.setShowQrcode(Preconditions.isNotBlank(thirdPlatformRespDto.getAdvertise()) ? Boolean.TRUE : Boolean.FALSE);
            //开门信号格式
            userVO.setIsOpen(Preconditions.isNotBlank(thirdPlatformRespDto.getIsOpen()) ? thirdPlatformRespDto.getIsOpen() : userVO.getIsOpen());
            if (Preconditions.isNotBlank(thirdPlatformRespDto.getSignalType())) {
                userVO.setUseSerialPort(thirdPlatformRespDto.getSignalType());
            }
            if (Preconditions.isNotBlank(thirdPlatformRespDto.getOpenTimes())) {
                userVO.setOpenTimes(thirdPlatformRespDto.getOpenTimes());
            }
            if (Preconditions.isNotBlank(thirdPlatformRespDto.getSignalNo())) {
                userVO.setSignalNo(thirdPlatformRespDto.getSignalNo());
            }
            return;
        }

    }

    private void convertCommonThirdOpenReturn(HealtCodeUserVO userVO, ThirdPlatformRespDto thirdPlatformRespDto, PlaceInfoVO placeInfoVO) {

        //头部
        if (Preconditions.isNotBlank(thirdPlatformRespDto)
                && Preconditions.isNotBlank(thirdPlatformRespDto.getMainLine())) {
            userVO.setTitle(thirdPlatformRespDto.getMainLine());
        }

        //通行结果
        placeInfoVO.setSubLineSuccess(Preconditions.isNotBlank(thirdPlatformRespDto.getSubLine()) ? thirdPlatformRespDto.getSubLine() : placeInfoVO.getSubLineSuccess());
        placeInfoVO.setSubLineFail(Preconditions.isNotBlank(thirdPlatformRespDto.getSubLine()) ? thirdPlatformRespDto.getSubLine() : placeInfoVO.getSubLineFail());
        placeInfoVO.setOpenVoiceMsgSuccess(Preconditions.isNotBlank(thirdPlatformRespDto.getOpenVoiceMsg()) ? thirdPlatformRespDto.getOpenVoiceMsg() : placeInfoVO.getOpenVoiceMsgSuccess());
        placeInfoVO.setOpenVoiceMsgFail(Preconditions.isNotBlank(thirdPlatformRespDto.getOpenVoiceMsg()) ? thirdPlatformRespDto.getOpenVoiceMsg() : placeInfoVO.getOpenVoiceMsgFail());

        //中下左部
        HealtCodeNucleinVO v = userVO.getNuclein().get(0);
        v.setTitle(Preconditions.isNotBlank(thirdPlatformRespDto.getMainLineLeft()) ? thirdPlatformRespDto.getMainLineLeft() : v.getTitle());
        v.setContent(Preconditions.isNotBlank(thirdPlatformRespDto.getSubLineLeft()) ? thirdPlatformRespDto.getSubLineLeft() : v.getContent());
        userVO.setNuclein(Arrays.asList(v));
        //中下右部
        userVO.setVaccineIcoTitle(Preconditions.isNotBlank(thirdPlatformRespDto.getMainLineRight()) ? thirdPlatformRespDto.getMainLineRight() : userVO.getVaccineIcoTitle());
        userVO.setVaccineNum(Preconditions.isNotBlank(thirdPlatformRespDto.getSubLineRight()) ? thirdPlatformRespDto.getSubLineRight() : userVO.getVaccineNum());
        //底部
        userVO.setQrcodeText(Preconditions.isNotBlank(thirdPlatformRespDto.getQrcodeText()) ? thirdPlatformRespDto.getQrcodeText() : userVO.getQrcodeText());
        userVO.setAdvertise(Preconditions.isNotBlank(thirdPlatformRespDto.getAdvertise()) ? thirdPlatformRespDto.getAdvertise() : userVO.getAdvertise());
        userVO.setShowQrcode(Preconditions.isNotBlank(thirdPlatformRespDto.getAdvertise()) ? Boolean.TRUE : Boolean.FALSE);

        userVO.setIsOpen(Preconditions.isNotBlank(thirdPlatformRespDto.getIsOpen()) ? thirdPlatformRespDto.getIsOpen() : userVO.getIsOpen());
        if (Preconditions.isNotBlank(thirdPlatformRespDto.getSignalType())) {
            userVO.setUseSerialPort(thirdPlatformRespDto.getSignalType());
        }
        if (Preconditions.isNotBlank(thirdPlatformRespDto.getOpenTimes())) {
            userVO.setOpenTimes(thirdPlatformRespDto.getOpenTimes());
        }
        if (Preconditions.isNotBlank(thirdPlatformRespDto.getSignalNo())) {
            userVO.setSignalNo(thirdPlatformRespDto.getSignalNo());
        }


    }

    private void handleQutong(PlaceInfoVO placeInfoVO, HealtCodeUserVO userVO, ShortTermDTO shortTermDTO, String openMsg) {
        if ((userVO.getIsOpen() == null || userVO.getIsOpen().intValue() == 0)
                && Preconditions.isNotBlank(placeInfoVO.getAdminId())
                && Preconditions.isNotBlank(quTongCodeServer.getAdminId())
                && quTongCodeServer.getAdminId().contains(placeInfoVO.getAdminId())) {
            QuTongIdentityAuthDto dto = new QuTongIdentityAuthDto();
            dto.setName(userVO.getUsername());
            dto.setMobile(userVO.getPhone());
            dto.setIdCard(userVO.getIdNumber());
            dto.setDeviceId(shortTermDTO.getSn());
            RestResult<QuTongIdentityVO> result = quTongCodeServer.identityAuth(dto);
            if (Preconditions.isBlank(result)) {
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                openMsg = "衢通码服务异常，请联系工作人员";
                placeInfoVO.setOpenVoiceMsgFail(openMsg);
                placeInfoVO.setSubLineFail(openMsg);
            } else if (RespCodeEnum.SUCCESS.getCode().intValue() != result.getCode()) {
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                openMsg = RespCodeEnum.getByCode(result.getCode()).getDesc();
                placeInfoVO.setOpenVoiceMsgFail(openMsg);
                placeInfoVO.setSubLineFail(openMsg);
            } else if (RespCodeEnum.SUCCESS.getCode().intValue() == result.getCode()) {
                openMsg = RespCodeEnum.getByCode(result.getCode()).getDesc();
                placeInfoVO.setOpenVoiceMsgSuccess(openMsg);
                placeInfoVO.setSubLineSuccess(openMsg);
            }
        }
    }

    private void handleScenic(PlaceInfoVO placeInfoVO, HealtCodeUserVO userVO, String openMsg, String openVoiceMsg) {
        if (Preconditions.isNotBlank(placeInfoVO)
                && placeInfoVO.getScenicFlag()
                && Preconditions.isNotBlank(userVO.getIdNumber())
                && IdcardUtil.getAgeByIdCard(userVO.getIdNumber()) < AGE) {
            //判断是否小于65周岁查询预约放行权限
            ScenicVerifyVO verify = scenicServer.verify(new ScenicVerifyDTO().setIdNumber(userVO.getIdNumber()), placeInfoVO.getAdminId());
            if (Preconditions.isNotBlank(verify) && !verify.getReleaseStatus()) {
//                未预约 请您先预约
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                openMsg = verify.getMessage();
                openVoiceMsg = openMsg;
            }
        }


    }

//    public static void main(String[] args) {
//        String value = "{\"aBackgroundUrl\":\"\",\"aLineOneLabel\":\"\",\"aLineOneValue\":\"\",\"aLineThrLabel\":\"\",\"aLineThrValue\":\"\",\"aLineTwoLabel\":\"\",\"aLineTwoValue\":\"\",\"aMainIcon\":0,\"aMainLine\":\"\",\"aSubLineFirst\":\"\",\"aSubLineSecond\":\"\",\"advertise\":\"\",\"countDown\":0,\"cChoiseList\":[{\"title\":\"陈鲁周\",\"subTitle\":\"320502199005113019\",\"list\":[{\"label\":\"耦园免费预约票\",\"value\":\"CT230904898754328730\",\"selected\":true}]}],\"bChoiseList\":null,\"bBackgroundUrl\":\"\",\"bMainIcon\":0,\"bMainLine\":\"\",\"bSubLine\":\"\",\"isOpen\":1,\"mainLine\":\"\",\"mainLineLeft\":\"\",\"mainLineRight\":\"\",\"openTimes\":0,\"openVoiceMsg\":\"\",\"qrcodeText\":\"\",\"showPageType\":\"scenic\",\"signalNo\":\"\",\"signalType\":\"\",\"status\":\"pendingMultiple\",\"taskId\":\"c66afde4-5d16-46b7-8b9d-8e69fb33b8e9\",\"subLine\":\"\",\"subLineLeft\":\"\",\"subLineRight\":\"\",\"end\":true,\"delay\":500}";
//        ThirdPlatformRespDto thirdPlatformRespDto = JSON.parseObject(value, ThirdPlatformRespDto.class);
//        log.info("-->:{}", JSONObject.toJSONString(thirdPlatformRespDto));
//
//    }


}
