package com.yixin.ms.server;

import cn.hutool.core.util.IdUtil;
import cn.hutool.crypto.symmetric.SM4;
import cn.hutool.crypto.symmetric.SymmetricCrypto;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yixin.ms.boot.core.Constants;
import com.yixin.ms.boot.restful.RestCode;
import com.yixin.ms.boot.restful.RestResponse;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.boot.uitls.Preconditions;
import com.yixin.ms.boot.uitls.RedisUtil;
import com.yixin.ms.boot.uitls.SMSOrIdCardUtils;
import com.yixin.ms.crypto.util.SzhzSm2Util;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.enums.HealthOpenEnum;
import com.yixin.ms.model.enums.IDTypeEnum;
import com.yixin.ms.model.enums.RestCodeEnum;
import com.yixin.ms.server.dto.DeviceScanCodeDTO;
import com.yixin.ms.server.dto.DeviceScanIdCardDTO;
import com.yixin.ms.server.dto.PlaceRecordDTO;
import com.yixin.ms.server.dto.TokenDataDTO;
import com.yixin.ms.server.vo.DeviceScanIdCardVO;
import com.yixin.ms.server.vo.HealthResultVO;
import com.yixin.ms.server.vo.PlaceInfoVO;
import com.yixin.ms.server.vo.QueryExamUserWhitelistVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import static com.yixin.ms.model.enums.PlaceAreaEnum.getByPlaceAreaCode;

/**
 * @author: lileilei
 * @date: 2022/6/23 10:47
 * @description: 杭州“四码一屏”统一接口
 */
@Configuration
@Slf4j
public class PlaceHealthCodeServer {

    @Value("${health.code.getTokenUrl}")
    private String getTokenUrl;

    @Value("${health.code.zhePublicKey}")
    private String zhePublicKey;

    @Value("${health.code.jiaPublicKey}")
    private String jiaPublicKey;

    @Value("${health.code.shaoPublicKey}")
    private String shaoPublicKey;

    @Value("${health.code.zheDomain}")
    private String zheDomain;

    @Value("${health.code.jiaDomain}")
    private String jiaDomain;

    @Value("${health.code.shaoDomain}")
    private String shaoDomain;

    @Value("${health.code.deviceScanIdCardUri}")
    private String deviceScanIdCardUri;

    @Value("${health.code.deviceScanIdCardEncryptUri}")
    private String deviceScanIdCardEncryptUri;

    @Value("${health.code.deviceScanIdCardEncryptBodyUri}")
    private String deviceScanIdCardEncryptBodyUri;

    @Value("${health.code.deviceScanCodeUri}")
    private String deviceScanCodeUri;

    @Value("${health.code.queryPlaceInfoBySnUrl}")
    private String queryPlaceInfoBySnUrl;
    @Value("${health.code.openPlaceInfoCache}")
    private Boolean openPlaceInfoCache;

    @Value("${health.code.savePlaceRecordUrl}")
    private String savePlaceRecordUrl;

    @Value("${health.code.queryUserExamInfo}")
    private String queryUserExamInfo;

    @Value("${health.code.checkIotUserPass}")
    private String checkIotUserPass;

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private RedisUtil redisUtil;

    /**
     * 获取token
     *
     * @return
     */
    public HealthResultVO<TokenDataDTO> getToken(Integer placeAreaCode) {
        return getToken(false, placeAreaCode);
    }

    /**
     * 获取token
     *
     * @return
     */
    public HealthResultVO<TokenDataDTO> getToken(Boolean cleanSession, Integer placeAreaCode) {
        HealthResultVO healthResultVO = restTemplate.getForObject(new StringBuilder().append(getTokenUrl).append("?cleanSession=").append(cleanSession).append("&placeAreaCode=").append(Preconditions.isBlank(placeAreaCode) ? 0 : placeAreaCode).toString(), HealthResultVO.class);
        log.info("获取token响应内容：{}", JSON.toJSONString(healthResultVO));
        healthResultVO.setData(JSON.parseObject(JSON.toJSONString(healthResultVO.getData()), TokenDataDTO.class));
        return healthResultVO;
    }


    /**
     * 扫人员证件查询场所码信息
     *
     * @return
     */
//    public HealthResultVO<DeviceScanIdCardVO> deviceScanIdCard(DeviceScanIdCardDTO dto) {
//        HealthResultVO healthResultVO = null;
//        try {
//            //认证token
//            String token = this.getToken(dto.getPlaceAreaCode()).getData().getToken();
//            dto.setToken(token);
//            dto.setBizNo(IdUtil.fastUUID());
//            String secret32Str = SecureUtil.md5(clientSecret).toLowerCase();
//            AES aes = SecureUtil.aes(secret32Str.getBytes());
//            String idNum = aes.encryptBase64(dto.getIdNumber().getBytes());
//            dto.setIdNumber(idNum);
//            healthResultVO = restTemplate.postForObject(deviceScanIdCardUrl, dto, HealthResultVO.class);
//            log.info("扫人员证件查询场所码信息返回内容: {}", JSON.toJSONString(healthResultVO));
//            healthResultVO.setData(JSON.parseObject(JSON.toJSONString(healthResultVO.getData()), DeviceScanIdCardVO.class));
//        } catch (Exception e) {
//            log.error("扫人员证件查询场所码信息异常：{}", e.getMessage());
//            e.printStackTrace();
//            return healthResultVO;
//        }
//        return healthResultVO;
//    }
    public HealthResultVO<DeviceScanIdCardVO> deviceScanIdCardEncrypt(ShortTermDTO shortTermDTO, PlaceInfoVO placeInfo) {
        DeviceScanIdCardDTO scanIdCardDTO = new DeviceScanIdCardDTO().setDeviceId(shortTermDTO.getSn())
                .setIdNumber(shortTermDTO.getIdNumber())
                .setPlaceAreaCode(Preconditions.isNotBlank(placeInfo.getPlaceAreaCode()) ? placeInfo.getPlaceAreaCode() : 0);
        if (SMSOrIdCardUtils.isIdCard(shortTermDTO.getIdNumber())) {
            scanIdCardDTO.setIdType(IDTypeEnum.ID_CARD.getCode());
        } else {
            scanIdCardDTO.setIdType(IDTypeEnum.PASSPORT.getCode());
        }
        scanIdCardDTO.setCityCode(getByPlaceAreaCode(placeInfo.getPlaceAreaCode()).getCityCode());

        //扫人员证件查询场所码信息
        HealthResultVO<DeviceScanIdCardVO> scanIdCardResult = getDeviceScanIdCardEncrypt(scanIdCardDTO);
        //换绑场所码或者管理员健康码首次会提示设备未打卡等错误，重新发起调用
        if (scanIdCardResult == null || !scanIdCardResult.getCode().equals(RestCode.SUCCESS.getCode())) {
            if (scanIdCardResult != null && scanIdCardResult.getCode().equals(RestCode.INVALID_TOKEN.getCode())) {
                //清理token缓存
                getToken(true, placeInfo.getPlaceAreaCode());
            }
            try {
                Thread.sleep(500);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            scanIdCardResult = getDeviceScanIdCardEncrypt(scanIdCardDTO);
        }
        return scanIdCardResult;
    }

    public HealthResultVO<DeviceScanIdCardVO> getDeviceScanIdCardEncrypt(DeviceScanIdCardDTO dto) {
        HealthResultVO healthResultVO = null;
        switch (getByPlaceAreaCode(dto.getPlaceAreaCode())) {
            case ZHE_JIANG:
                //浙江
                healthResultVO = deviceScanIdCardEncrypt(dto);
                break;
            default:
                healthResultVO = deviceScanIdCardEncryptBody(dto);
                break;
        }
        return healthResultVO;
    }

    /**
     * 扫人员证件查询场所码信息(请求体加密)
     *
     * @return
     */
    private HealthResultVO<DeviceScanIdCardVO> deviceScanIdCardEncrypt(DeviceScanIdCardDTO dto) {
        HealthResultVO healthResultVO = null;
        try {
            //认证token
            String token = this.getToken(dto.getPlaceAreaCode()).getData().getToken();
            dto.setToken(token);
            dto.setBizNo(IdUtil.fastUUID());
            dto.setIdNumber(dto.getIdNumber());
            String jsonReq = JSON.toJSONString(dto);
            //得到加密的jsonBody,公钥在文档中有，或者找数杭接口人获取
            String requestBody = SzhzSm2Util.encrypt(getPublicKey(dto.getPlaceAreaCode()), jsonReq);
            //获取最终的传参的JSON串
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("scanIdCardBody", requestBody);
            String domain = getDomain(dto.getPlaceAreaCode());
            healthResultVO = restTemplate.postForObject(domain + deviceScanIdCardEncryptUri, jsonObject, HealthResultVO.class);
            log.info("扫人员证件查询场所码信息返回内容: {}", JSON.toJSONString(healthResultVO));
            healthResultVO.setData(JSON.parseObject(JSON.toJSONString(healthResultVO.getData()), DeviceScanIdCardVO.class));
        } catch (Exception e) {
            log.error("扫人员证件查询场所码信息异常：{}", e.getMessage());
            e.printStackTrace();
            return healthResultVO;
        }
        return healthResultVO;
    }

    /**
     * 扫人员证件查询场所码信息(请求体加密) SM4国密加密
     *
     * @return
     */
    private HealthResultVO<DeviceScanIdCardVO> deviceScanIdCardEncryptBody(DeviceScanIdCardDTO dto) {
        HealthResultVO healthResultVO = null;
        try {
            //认证token
            String token = this.getToken(dto.getPlaceAreaCode()).getData().getToken();
            dto.setToken(token);
            dto.setBizNo(IdUtil.fastUUID());
            dto.setIdNumber(dto.getIdNumber());
            String jsonReq = JSON.toJSONString(dto);
            // 得到加密的jsonBody， 公钥在文档中有，或者找数杭接口人获取
            SymmetricCrypto symmetricCrypto = new SM4("ecb", "PKCS7Padding", getPublicKey(dto.getPlaceAreaCode()).getBytes());
            String encryptBody = symmetricCrypto.encryptHex(jsonReq);
            // 获取最终的传参的JSON串
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("encryptBody", encryptBody);
            jsonObject.put("token", token);
            // 最终传参的JSON串 文档有误不能传递string
//            String finalJsonReq = jsonObject.toJSONString();
            String domain = getDomain(dto.getPlaceAreaCode());
            healthResultVO = restTemplate.postForObject(domain + deviceScanIdCardEncryptBodyUri, jsonObject, HealthResultVO.class);
            log.info("--->国密SM4扫人员证件查询场所码信息返回内容: {}", JSON.toJSONString(healthResultVO));
            healthResultVO.setData(JSON.parseObject(JSON.toJSONString(healthResultVO.getData()), DeviceScanIdCardVO.class));
        } catch (Exception e) {
            log.error("--->国密SM4扫人员证件查询场所码信息异常：{}", e.getMessage());
            e.printStackTrace();
            return healthResultVO;
        }
        return healthResultVO;
    }

    private String getPublicKey(Integer placeAreaCode) {
        String publicKey = zhePublicKey;
        if (Preconditions.isNotBlank(placeAreaCode)) {
            switch (getByPlaceAreaCode(placeAreaCode)) {
                case JIA_XIN:
                    //嘉兴
                    publicKey = jiaPublicKey;
                    break;
                case SHAO_XING:
                    //绍兴
                    publicKey = shaoPublicKey;
                    break;
                default:
                    //浙江
                    break;
            }
        }
        return publicKey;
    }

    private String getDomain(Integer placeAreaCode) {
        String domain = zheDomain;
        if (Preconditions.isNotBlank(placeAreaCode)) {
            switch (getByPlaceAreaCode(placeAreaCode)) {
                case JIA_XIN:
                    //嘉兴
                    domain = jiaDomain;
                    break;
                case SHAO_XING:
                    //绍兴
                    domain = shaoDomain;
                    break;
                default:
                    //浙江
                    break;
            }
        }
        return domain;
    }

    public HealthResultVO<DeviceScanIdCardVO> deviceScanCode(ShortTermDTO shortTermDTO, PlaceInfoVO placeInfo) {
        //健康码查询场所码信息
        HealthResultVO<DeviceScanIdCardVO> scanIdCardResult = this.deviceScanCode(new DeviceScanCodeDTO()
                .setDeviceId(shortTermDTO.getSn()).setHealthCodeQrCode(shortTermDTO.getHealthCode())
                .setUrlParams(shortTermDTO.getHealthCode())
                .setPlaceAreaCode(Preconditions.isNotBlank(placeInfo.getPlaceAreaCode()) ? placeInfo.getPlaceAreaCode() : 0));
        //换绑场所码或者管理员健康码首次会提示设备未打卡等错误，重新发起调用
        if (scanIdCardResult == null || !scanIdCardResult.getCode().equals(RestCode.SUCCESS.getCode())) {
            if (scanIdCardResult != null && scanIdCardResult.getCode().equals(RestCode.INVALID_TOKEN.getCode())) {
                //清理token缓存
                this.getToken(true, placeInfo.getPlaceAreaCode());
            }
            try {
                Thread.sleep(500);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            scanIdCardResult = this.deviceScanCode(new DeviceScanCodeDTO()
                    .setDeviceId(shortTermDTO.getSn()).setHealthCodeQrCode(shortTermDTO.getHealthCode())
                    .setUrlParams(shortTermDTO.getHealthCode())
                    .setPlaceAreaCode(Preconditions.isNotBlank(placeInfo.getPlaceAreaCode()) ? placeInfo.getPlaceAreaCode() : 0));
        }

        return scanIdCardResult;
    }

    /**
     * 健康码查询场所码信息
     * <p>
     * 根据barcode查询用户健康码信息（四码一屏）
     *
     * @return
     */
    public HealthResultVO<DeviceScanIdCardVO> deviceScanCode(DeviceScanCodeDTO dto) {
        HealthResultVO healthResultVO = null;
        try {
            //认证token
            String token = this.getToken(dto.getPlaceAreaCode()).getData().getToken();
            dto.setToken(token);
            dto.setBizNo(IdUtil.fastUUID());
            String domain = getDomain(dto.getPlaceAreaCode());
            healthResultVO = restTemplate.postForObject(domain + deviceScanCodeUri, dto, HealthResultVO.class);
            log.info("健康码查询场所码信息返回内容: {}", JSON.toJSONString(healthResultVO));
            healthResultVO.setData(JSON.parseObject(JSON.toJSONString(healthResultVO.getData()), DeviceScanIdCardVO.class));
        } catch (Exception e) {
            log.error("健康码查询场所码信息：{}", e.getMessage());
            e.printStackTrace();
            return healthResultVO;
        }
        return healthResultVO;
    }

    /**
     * 通过设备sn查询账户信息
     */
    public PlaceInfoVO queryAdminInfoBySn(String sn) {
        PlaceInfoVO placeInfoVO = null;
        if (Preconditions.isNotBlank(sn)) {
            String snConfigKey = Constants.REDIS_SN_DATA_NAME + sn;
            if(openPlaceInfoCache){
                //先获取缓存的
                if(redisUtil.hasKey(snConfigKey)){
                    placeInfoVO = redisUtil.getObject(snConfigKey);
                }
            }
            if(placeInfoVO == null){
                RestResult result = restTemplate.getForObject(new StringBuilder().append(queryPlaceInfoBySnUrl).append("?sn=").append(sn).toString(), RestResult.class);
                placeInfoVO = JSON.parseObject(JSON.toJSONString(result.getData()), PlaceInfoVO.class);
                log.info("通过设备sn: {}, 查询账户信息返回内容:{}", sn, JSON.toJSONString(result));
                if (result.getCode() == RestCodeEnum.DEVICE_NOT_FOUND.getCode()) {
                    //设备未注册 未录入系统
                    throw new ServiceException(RestCodeEnum.DEVICE_NOT_FOUND.getMsg());
                }
                if (result.getCode() == RestCodeEnum.UNAUTHORIZED.getCode()) {
                    //设备未注册 未录入系统
                    throw new ServiceException("设备未授权");
                }
                redisUtil.set(snConfigKey,placeInfoVO,3600 * 36);
            }
        }
        if (Preconditions.isBlank(placeInfoVO)) {
            throw new ServiceException(RestCodeEnum.DEVICE_NOT_FOUND.getMsg());
        }
        return placeInfoVO;
    }
    public void getSnConfig(RestResult<PlaceInfoVO> rest) {
        if (rest.getCode() == RestCodeEnum.SUCCESS.getCode()){
            PlaceInfoVO placeInfoVO = rest.getData();
            String snConfigKey = Constants.REDIS_SN_DATA_NAME + placeInfoVO.getSn();
            redisUtil.set(snConfigKey,placeInfoVO,3600 * 36);
        }else {
            String snConfigKey = Constants.REDIS_SN_DATA_NAME + rest.getMsg();//在code不是200的时候 msg返回设备号
            redisUtil.delete(snConfigKey);//清除缓存
        }
    }

    /**
     * 保存场所记录
     */
    public RestResult<PlaceInfoVO> savePlaceRecord(PlaceRecordDTO dto) {
        return restTemplate.postForObject(savePlaceRecordUrl, dto, RestResult.class);
    }

    /**
     * 获取考试信息
     *
     * @param sn
     * @param idNumber
     * @return
     */
    public RestResult<QueryExamUserWhitelistVO> getExamUserWhitelist(String sn, String idNumber) {
        RestResult result = null;
        try {
            JSONObject object = new JSONObject();
            object.put("sn", sn);
            object.put("idNumber", idNumber);
            result = restTemplate.postForObject(queryUserExamInfo, object, RestResult.class);
            log.info("通过sn:{},idNumber:{},查询用户考试信息返回内容:{}", sn, idNumber, JSON.toJSONString(result));
            result.setData(JSON.parseObject(JSON.toJSONString(result.getData()), QueryExamUserWhitelistVO.class));
        } catch (Exception e) {
            log.error("查询用户考试信息异常：{}", e.getMessage());
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 校验用户是否在iot设备
     *
     * @param idNumber
     * @param sn
     */
    public RestResult checkIotUserPass(String idNumber, String sn) {
        RestResult result = RestResponse.error(HealthOpenEnum.NO_THOROUGHFARE.getDesc());
        try {
            JSONObject object = new JSONObject();
            object.put("sn", sn);
            object.put("idNumber", idNumber);
            result = restTemplate.postForObject(checkIotUserPass, object, RestResult.class);
            log.info("通过sn:{},idNumber:{},校验用户是否在iot设备返回内容:{}", sn, idNumber, JSON.toJSONString(result));
        } catch (Exception e) {
            log.error("校验用户是否在iot设备异常：{}", e.getMessage());
            e.printStackTrace();

        }
        return result;
    }

}
