package com.yixin.ms.server;

import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.server.dto.QuTongIdentityAuthDto;
import com.yixin.ms.server.vo.QuTongIdentityVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author shisl
 * @package com.yixin.ms.server
 * @class QuTongCodeServer
 * @date 2023/4/12 上午9:17
 * @description 衢通码
 */
@Slf4j
@Component
public class QuTongCodeServer {

    @Value("${qu.tong.domain}")
    private String domain;

    @Value("${qu.tong.identityAuthUri}")
    private String identityAuthUri;

    @Value("${qu.tong.deviceType}")
    private Integer deviceType;

    @Value("${qu.tong.adminId}")
    private String adminId;

    /**
     * IOT设备身份认证
     *
     * @param dto
     * @return
     */
    public RestResult<QuTongIdentityVO> identityAuth(QuTongIdentityAuthDto dto) {
        RestResult result = null;
        try {
            dto.setDeviceType(deviceType);
            String body = HttpUtil.post(domain + identityAuthUri, JSON.toJSONString(dto));
            log.info("IOT设备身份认证，入参信息：{}, 响应信息：{}", JSON.toJSONString(dto), body);
            result = JSON.parseObject(body, RestResult.class);
            result.setData(JSON.parseObject(JSON.toJSONString(result.getData()), QuTongIdentityVO.class));
        } catch (Exception e) {
            log.error("IOT设备身份认证异常：{}", e.getMessage());
            e.printStackTrace();
        }
        return result;
    }

    public List<Integer> getAdminId(){
        List<Integer> adminIds = Arrays.asList(adminId.split(",")).stream().map(item -> Integer.parseInt(item)).collect(Collectors.toList());
        return adminIds;
    }

    public static void main(String[] args) {
        QuTongIdentityAuthDto dto = new QuTongIdentityAuthDto();
        dto.setDeviceId("21CF010T00100213");
        dto.setDeviceType(217583);
        dto.setIdCard("330481199704031831");
        dto.setMobile("13515834834");
        dto.setName("王鑫刚");
        String body = HttpUtil.post("https://xasz.0570w.net/visitor-code/iot/api/identityAuthentication", JSON.toJSONString(dto));
        RestResult result = JSON.parseObject(body, RestResult.class);
        result.setMsg(QuTongIdentityVO.RespCodeEnum.getByCode(result.getCode()).getDesc());
        log.info("IOT设备身份认证，入参信息：{}, 响应信息：{}", JSON.toJSONString(dto), JSON.toJSONString(result));
    }


}
